#ifndef _UNWINDOWS_H
#define _UNWINDOWS_H

#if defined(__GLIBC__) && !defined(_GNU_SOURCE)
#    define _GNU_SOURCE
#endif

#if defined(__cplusplus) || (__STDC_VERSION__ == 199901L)
#    define INLINE inline
#else
#    define INLINE
#endif

#ifdef WIN32
/*******************************************************
 * WINDOWS STUFF
 *******************************************************/
#   if !defined(NICE_AND_BLOATED) && !defined(WIN32_LEAN_AND_MEAN)
#       define WIN32_LEAN_AND_MEAN 1
#   endif
#   include <windows.h>
#   include <string.h>
#   include <stdlib.h>
#   include <stdio.h>
#   include <stdarg.h>
#   include <math.h>

#   define strcasecmp  stricmp
#   define strncasecmp strnicmp
#   define snprintf    _snprintf
#   define vsnprintf   _vsnprintf

#   define sleep       Sleep

#   define _DEV_NULL_STR "nul"

#   include <float.h>
#   define isnan       _isnan
    typedef union { DWORD64 q; double d; } _unwindows_dw64qdbld;
    typedef union { DWORD32 l; float f;  } _unwindows_dw32lfltf;
#   define ASSIGN_NAN_DBL(dout)           \
    {                                     \
        _unwindows_dw64qdbld _uw_nan;     \
        _uw_nan.q = 0x7ff8000000000000;   \
        (dout) = _uw_nan.d;               \
    }
#   define ASSIGN_NAN_FLT(fout)           \
    {                                     \
        _unwindows_dw32lfltf _uw_nan;     \
        _uw_nan.l = 0x7fc00000;           \
        (fout) = _uw_nan.f;               \
    }

#else
/*******************************************************
 * NON-WINDOWS STUFF
 *******************************************************/
#   define _DEV_NULL_STR "/dev/null"

#   include <string.h>
#   include <stdlib.h>
#   include <stdio.h>
#   include <stdarg.h>
#   include <math.h>

#   include <sys/time.h>
    static INLINE unsigned long GetTickCount(void)
    {
        struct timeval time;
        gettimeofday(&time, NULL);
        return 1000*time.tv_sec + time.tv_usec/1000;
    }

#   define ASSIGN_NAN_DBL(d) { (d) = (double)NAN; }
#   define ASSIGN_NAN_FLT(f) { (f) = (float)NAN; }

#endif

#if !defined(_GNU_SOURCE) && (__STDC_VERSION__ != 199901L)
static INLINE int vasprintf(char **strp, const char *fmt, va_list ap)
{
    FILE *dev_null;
    int arg_len;
    
    dev_null    = fopen(_DEV_NULL_STR, "w");
    arg_len     = vfprintf(dev_null, fmt, ap);
    if (arg_len != -1) {
        *strp   = (char *)calloc((size_t)arg_len + 1, sizeof(char));
        arg_len = vsprintf(*strp, fmt, ap);
    }
    fclose(dev_null);
    return arg_len;
}

static INLINE int asprintf(char **strp, const char *fmt, ...)
{
    int result;
    va_list args;
    va_start(args, fmt);
    result = vasprintf(strp, fmt, args);
    va_end(args);
    return result;
}
#endif

#undef INLINE
#endif
